/*****************************************************************************/
/*                                                                           */
/*  NetReconstruct kappa version                                             */
/*                                                                           */
/*  Software to perform phylogenetic network reconstructions when            */
/*  diploid hybrids are part of the evolutionary history.                    */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#ifndef _GLOBALS_H
#define _GLOBALS_H

/***************************/
/*     global constants    */
/***************************/

#define DNA_DIST_CUSTOM_FN_STR_LENGTH 200
#define MAX_INFILE_NAME_LENGTH 200
#define MAX_LEAVES 100
#define MAX_NODES 2500
#define MAX_NODE_NAME 11 // limit for phylip is 10 so add null and that's it
#define MAX_NUM_HOM_CHROMOS 10
#define MAX_NUM_STRANDS_PER_HC 8
#define MAX_NUM_HYBRIDS 10
#define MAX_SEQ_LENGTH 2000
#define MAX_TOKEN 100

enum {L_PAREN, L_PAREN_TAXON_NAME, COMMA_L_PAREN, COMMA_TAXON_NAME, R_PAREN, R_PAREN_TAXON_NAME, POUND, COLON, SEMI_COLON, EOL, ILLEGAL_CHAR, UNKNOWN};

/***************************/
/*        defaults         */
/***************************/

#define MAX_LINE_LENGTH 5000
#define MP_PHYLIP_THRESH_DEFAULT 9
#define PARENTAL_AFF_DEFAULT -1


/********************/
/*     variables    */
/********************/

// These are input values from the user that are needed
// all over the place and central structures

extern char dna_dist_custom_file_name[DNA_DIST_CUSTOM_FN_STR_LENGTH];
extern char infile_name[MAX_INFILE_NAME_LENGTH];
extern char token_char[MAX_TOKEN];

extern long input_seed;

extern int strand_summed_hd;
extern int dna_dist_model; // 0=jukes-cantor, 1=f84, 2=k2p, 3=logdet, 4=custom
extern int dna_dist_model_yn; // 0 not specified, 1 it is specified
extern int extant_taxa_read;
extern int hyb_parent_hit_ctr;
extern int last_hyb_parent_hit;
extern int mp_phylip_thresh;
extern int num_strands_per_hom_chrom;
extern int num_hom_chromos;
extern int num_extant_total;
extern int num_impacted_by_hybrid;
extern int num_not_impacted;
extern int num_impacted_by_parents;
extern int num_impacted_by_p1;
extern int num_impacted_by_p2;
extern int outgroup_adjustment;
extern int outgroup_taxon_id;
extern int parental_offspring_id_method;  // 0 = default closest_neighbor
                                          // 1 = closest_2x_hybrid
					  // 2 = custom
extern int recon_type;       // 0 = PHYLIP
                             // 1 = RANDIP
extern int sequence_length;
extern int subtree_listing_ctr;
extern int token_int;
extern int token_type;

extern float token_float;

extern FILE *fp_in; // global input
extern FILE *fp_temp_in; // local input
extern FILE *fp_out; // global output
extern FILE *fp_temp_out; // local output

#endif
