/*****************************************************************************/
/*                                                                           */
/*  NetReconstruct kappa version                                             */
/*                                                                           */
/*  Software to perform phylogenetic network reconstructions when            */
/*  diploid hybrids are part of the evolutionary history.                    */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "node.h"
#include "network.h"
#include "input.h"
#include "output.h"
#include "hybsub.h"
#include "extstruct.h"
#include "parsub.h"
#include "rem.h"
#include "finnet.h"
#include "twister.h"

/*****************************************************************************/
/*                                  main                                     */
/*****************************************************************************/

int main(const int argc, const char *argv[])
{
  NETWORK nt;

  /****************************************************************************/
  /*  the command line is:  'NR -i <inputfile> -o <outputfile> -r <randseed>' */
  /****************************************************************************/

  if (argc != 7)
  {
    printf("NR ERROR: Command line is:  'NR -i <inputfile> -o <outputfile> -r <randseed>'\n");
    exit(0);
  };

  if (strcmp(argv[1], "-i")!=0 || 
             strcmp(argv[3],"-o")!=0 || 
	     strcmp(argv[5], "-r")!=0)
  {
    printf("NR ERROR: Command line is:  'NR -i <inputfile> -o <outputfile> -r <randseed>'\n");
    exit(0);
  }

  strcpy(infile_name, argv[2]);

  // open the input/output files
  fp_in = fopen(argv[2], "r");
  if (fp_in == NULL)
  {
    printf("NR ERROR: Can't open the input file '%s'\n", argv[2]);
    exit(0);
  }

  fp_out = fopen(argv[4], "w");
  if (fp_out == NULL)
  {
    printf("NR ERROR: Can't open the output file '%s'\n", argv[4]);
    exit(0);
  }

  // set the seed for the random numbers
  sscanf(argv[6], "%lu", &input_seed);
  init_genrand(input_seed);

  // first make the basics of the network
  nt = NewNetwork();

  // read the input
  ReadInput(nt);

  // now do the important work
  ReconHybridSubtree(nt);
  ExtendStructure(nt);
  ReconParentalSubtree(nt);
  ReconRemainder(nt);
  FinishNetwork(nt);

  // also calc the parsimony score so we can report it
  CalcParsimonyScore(nt);

  // write the output
  NetworkReport(nt);
  
  // be good and clean up
  fclose(fp_in);
  fclose(fp_out);


  return (1);
}
