/*****************************************************************************/
/*                                                                           */
/*  NetReconstruct kappa version                                             */
/*                                                                           */
/*  Software to perform phylogenetic network reconstructions when            */
/*  diploid hybrids are part of the evolutionary history.                    */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<math.h>
#include"globals.h"
#include"node.h"
#include"network.h"
#include"output.h"


/******************************************************************************/
/*                              NetworkReport                                 */
/******************************************************************************/
void NetworkReport(NETWORK nt)
{
  SummaryStats(nt);
  InputtedExtantTaxa(nt);
  NetworkTopology(nt);
  ParsimonyScoreReport(nt);
  HybridHDHeightReport(nt);

  fprintf(fp_out, "\n\n\n");

  //AllTaxa(nt);

  return;
  
}  // end of NetworkReport

/******************************************************************************/
/*                                SummaryStats                                */
/******************************************************************************/
void SummaryStats(NETWORK nt)
{
  int i;
  
  fprintf(fp_out, "Summary Stats\n");

  fprintf(fp_out, "The input file for this run was %s \n", infile_name);
  fprintf(fp_out, "The input seed for this run was %ld \n", input_seed);

  fprintf(fp_out, "\n");

  if (recon_type == 0)
  {
    fprintf(fp_out, "PHYLIP was used for the three primary reconstruction phases.\n");
    fprintf(fp_out, "The threshold used to decide whether dnapenny or\n");
    fprintf(fp_out, "neighbor was used for reconstruction was %d\n", mp_phylip_thresh);

    fprintf(fp_out, "The evolutionary model for PHYLIP's dnadist routine (if called) was: ");
    if (dna_dist_model == 0)
    {
      fprintf(fp_out, "Jukes-Cantor\n");
    }
    else if (dna_dist_model == 1)
    {
      fprintf(fp_out, "F84\n");
    }
    else if (dna_dist_model == 2)
    {
      fprintf(fp_out, "K2P\n");
    }
    else if (dna_dist_model == 3)
    {
      fprintf(fp_out, "LogDet\n");
    }
    else if (dna_dist_model == 4)
    {
      fprintf(fp_out, "custom (with the file of: %s)\n", dna_dist_custom_file_name);
    }
  }
  else if (recon_type == 1)
  {
    fprintf(fp_out, "RANDIP was used for the three primary reconstruction phases -- this is typically used only for analysis.\n");
  }

  fprintf(fp_out, "\n");

  fprintf(fp_out, "The number of extant leaves inputted was : %d\n", nt->num_unique_extant_leaves);

  fprintf(fp_out, "The outgroup taxon id for the network: %d\n", outgroup_taxon_id);

  fprintf(fp_out, "\n");

  fprintf(fp_out, "Hybrid subtree(s) info:\n");
  
  for (i = 0; i < MAX_NUM_HYBRIDS; i++)
  {
    if (nt->outgroup_ids_for_subtrees[i] != -1)
    {
      fprintf(fp_out, "  subtree %d outgroup_id:%d   parental_offspring_id_method: ", i, nt->outgroup_ids_for_subtrees[i]);

      if (parental_offspring_id_method == 0)
        fprintf(fp_out, "default -- closest_neighbor\n");
      else if (parental_offspring_id_method == 1)
        fprintf(fp_out, "closest_2x_hybrid\n");
      else if (parental_offspring_id_method == 2)
        fprintf(fp_out, "custom\n");
      else if (parental_offspring_id_method == 3)
        fprintf(fp_out, "extreme custom\n");
      else
        fprintf(fp_out, "uknown type -- PROBLEM!\n");
    }
  }

  fprintf(fp_out, "\n");

  return;
}

/******************************************************************************/
/*                            InputtedExtantTaxa                              */
/******************************************************************************/
void InputtedExtantTaxa(NETWORK nt)
{
  int hchromo_i, strand_i, node_i, i;
  NODE nd;

  // starting the basics of the report
  fprintf(fp_out, "Inputted Extant Taxa\n");

  fprintf(fp_out, "\n");

  for (node_i = 0; node_i < nt->num_unique_nodes; node_i++)
  {
    nd = nt->NodeArray[node_i];

    if (nd->extant_leaf == 1)
    {
      if ( (nd->impacted_by_hybrid[0] == -1) &&
           (nd->impacted_by_parents[0] == -1) )
      {
        fprintf(fp_out, "id: %d\n", nd->external_node_id);
      }
      else
      {
        if (nd->impacted_by_hybrid[0] != -1)
	{
          fprintf(fp_out, "id: %d impacted by hybrid id(s): ", nd->external_node_id);
        
	  i = 0;

	  while ( (nd->impacted_by_hybrid[i] != -1) &&
	          (i < MAX_NUM_HYBRIDS) )
	  {
	    fprintf(fp_out, "h%d ", nd->impacted_by_hybrid[i]);
	    i++;
	  }

	  fprintf(fp_out, "\n");
	}

        if (nd->impacted_by_parents[0] != -1)
	{
          fprintf(fp_out, "id: %d impacted by the parents of hybrid id(s): ", nd->external_node_id);
	 
	  i = 0;

	  while ( (nd->impacted_by_parents[i] != -1) &&
	          (i < MAX_NUM_HYBRIDS) )
	  {
	    fprintf(fp_out, "h%d ", nd->impacted_by_parents[i]);

            if (nd->parental_aff == -2)
	      fprintf(fp_out, "(p1) ");
	    else if (nd->parental_aff == -3)
	      fprintf(fp_out, "(p2) ");

	    i++;
	  }

	  fprintf(fp_out, "\n");
	}
      }

      for (hchromo_i = 0; hchromo_i < num_hom_chromos; hchromo_i++)
      {
        for (strand_i = 0; strand_i < num_strands_per_hom_chrom; strand_i++)
        {
	  fprintf(fp_out, "%s\n", nd->hom_chromos[hchromo_i]->strands[strand_i]->sequence);
        }
      }

      fprintf(fp_out, "\n");
    }
  }

  return;

}

/******************************************************************************/
/*                            NetworkTopology                                 */
/******************************************************************************/
void NetworkTopology(NETWORK nt)
{
  int id_opt;
  int bl_opt;

 
  /*****************************/
  /*      internal ids         */
  /*****************************/

  id_opt = 1;
  bl_opt = 0;

  fprintf(fp_out, "The topology of the network (in modified newick format) using internal id numbers is:\n");

  if (nt->root == NULL)
  {
    fprintf(fp_out, "No topology to report, root is NULL");
  }
  else if (IsTree(nt)) // if tree, print regular newick format
  {
    ResetMarkersTopo(nt->root);
    PrintNewick(nt->root, id_opt, bl_opt);
    // termination for newick format
    fprintf(fp_out, ")%d;\n", nt->root->internal_node_id);
    ResetMarkersTopo(nt->root);
  }
  else  // is NOT a tree
  {
    ResetMarkersTopo(nt->root);
    last_hyb_parent_hit = 0;
    hyb_parent_hit_ctr = 0;
    PrintModifiedNewick(nt->root, id_opt, bl_opt);
    // termination for modified newick format
    fprintf(fp_out, ")%d;\n", nt->root->internal_node_id);
    ResetMarkersTopo(nt->root);
  }

  fprintf(fp_out, "\n");
  fprintf(fp_out, "\n");

  /*****************************/
  /*      external ids         */
  /*****************************/

  id_opt = 2;
  bl_opt = 0;

  fprintf(fp_out, "The topology of the network (in modified newick format) using external id numbers is:\n");

  if (nt->root == NULL)
  {
    fprintf(fp_out, "No topology to report, root is NULL");
  }
  else if (IsTree(nt)) // if tree, print regular newick format
  {
    ResetMarkersTopo(nt->root);
    PrintNewick(nt->root, id_opt, bl_opt);
    // termination for newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }
  else  // is NOT a tree
  {
    ResetMarkersTopo(nt->root);
    last_hyb_parent_hit = 0;
    hyb_parent_hit_ctr = 0;
    PrintModifiedNewick(nt->root, id_opt, bl_opt);
    // termination for modified newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }

  fprintf(fp_out, "\n");
  fprintf(fp_out, "\n");

  /******************************************************/
  /*      internal ids -- with HD as branch lengths     */
  /******************************************************/

  id_opt = 1;
  bl_opt = 1; 

  fprintf(fp_out, "The topology of the network (in modified newick format) using internal id numbers and hamming distances as branch lengths is:\n");

  if (nt->root == NULL)
  {
    fprintf(fp_out, "No topology to report, root is NULL");
  }
  else if (IsTree(nt)) // if tree, print regular newick format
  {
    ResetMarkersTopo(nt->root);
    PrintNewick(nt->root, id_opt, bl_opt);
    // termination for newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }
  else  // is NOT a tree
  {
    ResetMarkersTopo(nt->root);
    last_hyb_parent_hit = 0;
    hyb_parent_hit_ctr = 0;
    PrintModifiedNewick(nt->root, id_opt, bl_opt);
    // termination for modified newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }

  fprintf(fp_out, "\n");
  fprintf(fp_out, "\n");

  /******************************************************/
  /*      external ids -- with HD as branch lengths     */
  /******************************************************/

  id_opt = 2;
  bl_opt = 1; 

  fprintf(fp_out, "The topology of the network (in modified newick format) using external id numbers and hamming distances as branch lengths is:\n");

  if (nt->root == NULL)
  {
    fprintf(fp_out, "No topology to report, root is NULL");
  }
  else if (IsTree(nt)) // if tree, print regular newick format
  {
    ResetMarkersTopo(nt->root);
    PrintNewick(nt->root, id_opt, bl_opt);
    // termination for newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }
  else  // is NOT a tree
  {
    ResetMarkersTopo(nt->root);
    last_hyb_parent_hit = 0;
    hyb_parent_hit_ctr = 0;
    PrintModifiedNewick(nt->root, id_opt, bl_opt);
    // termination for modified newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }

  fprintf(fp_out, "\n");
  fprintf(fp_out, "\n");

  /*************************************************************/
  /*      internal ids -- with EVOLUTIONARY branch lengths     */
  /*************************************************************/

  id_opt = 1;
  bl_opt = 2; 

  fprintf(fp_out, "The topology of the network (in modified newick format) using internal id numbers and evolutionary branch lengths is:\n");

  if (nt->root == NULL)
  {
    fprintf(fp_out, "No topology to report, root is NULL");
  }
  else if (IsTree(nt)) // if tree, print regular newick format
  {
    ResetMarkersTopo(nt->root);
    PrintNewick(nt->root, id_opt, bl_opt);
    // termination for newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }
  else  // is NOT a tree
  {
    ResetMarkersTopo(nt->root);
    last_hyb_parent_hit = 0;
    hyb_parent_hit_ctr = 0;
    PrintModifiedNewick(nt->root, id_opt, bl_opt);
    // termination for modified newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }

  fprintf(fp_out, "\n");
  fprintf(fp_out, "\n");

  /*************************************************************/
  /*      external ids -- with EVOLUTIONARY branch lengths     */
  /*************************************************************/

  id_opt = 2;
  bl_opt = 2; 

  fprintf(fp_out, "The topology of the network (in modified newick format) using external id numbers and evolutionary branch lengths is:\n");

  if (nt->root == NULL)
  {
    fprintf(fp_out, "No topology to report, root is NULL");
  }
  else if (IsTree(nt)) // if tree, print regular newick format
  {
    ResetMarkersTopo(nt->root);
    PrintNewick(nt->root, id_opt, bl_opt);
    // termination for newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }
  else  // is NOT a tree
  {
    ResetMarkersTopo(nt->root);
    last_hyb_parent_hit = 0;
    hyb_parent_hit_ctr = 0;
    PrintModifiedNewick(nt->root, id_opt, bl_opt);
    // termination for modified newick format
    fprintf(fp_out, ")%d;\n", nt->root->external_node_id);
    ResetMarkersTopo(nt->root);
  }

  fprintf(fp_out, "\n");
  fprintf(fp_out, "\n");

  return;
}

/******************************************************************************/
/*                           ParsimonyScoreReport                             */
/******************************************************************************/
void ParsimonyScoreReport(NETWORK nt)
{

  fprintf(fp_out, "Parsimony score is calculated as the sum of changes each\n");
  fprintf(fp_out, "strand undergoes (as measured from node to node)\n");
  fprintf(fp_out, "throughout the network.  Reticulate node(s) and their\n");
  fprintf(fp_out, "descendants are only considered for the strands given\n");
  fprintf(fp_out, "to them.\n");

  fprintf(fp_out, "\n The parsimony score for this network is : %d\n", nt->parsimony_score);

  return;
}

/******************************************************************************/
/*                           HybridHDHeightReport                             */
/******************************************************************************/
void HybridHDHeightReport(NETWORK nt)
{
  NODE nd;
  NODE rt;
  NODE h;
  int node_i;
  int hd;
  int h_count;
  int e_count;
  int running_total;
  int h_hd_height;
  int n_hd_height;

  rt = nt->root;
  hd = 0;
  h_count = 0;
  e_count = 0;
  running_total = 0;

  // find what should be the single hybrid node
  for (node_i = 0; node_i < nt->num_unique_nodes; node_i++)
  {
    nd = nt->NodeArray[node_i];

    if ( (nd->temporary == 0) && (nd->reticulate == 1) ) 
    {
      h_count++;
      h = nd;

      if (h_count > 1)
      {
        printf("NR ERROR:  More than one hybrid node detected -- problem -- program aborted\n");
	exit(0);
      }
    }
  }

  // calculate the hamming distance between the hybrid node and the root now
  // that we have both of those nodes
  h_hd_height = HammingDistanceTotal(rt, h);
  
  // now we go through through and calculate the average hamming distance
  // for each extant and the root and use this as the network's hamming
  // distance height
  for (node_i = 0; node_i < nt->num_unique_nodes; node_i++)
  {
    nd = nt->NodeArray[node_i];

    if ( (nd->temporary == 0) && (nd->extant_leaf == 1) )
    {
      e_count++;

      hd = HammingDistanceTotal(rt, nd);

      //printf("NR DEBUG:  between root and int id %d was %d \n", nd->internal_node_id, hd);
      
      running_total = running_total + hd;
    }
  }

  n_hd_height = running_total / e_count;


  // print the results
  fprintf(fp_out, "\nThe HD height for the hybrid node is: %d  and the average for the network itself is: %d \n", h_hd_height, n_hd_height);

  return;
}


/******************************************************************************/
/*                                AllTaxa                                     */
/******************************************************************************/
void AllTaxa(NETWORK nt)
{
  int hchromo_i, strand_i, node_i;
  NODE nd;

  // starting the basics of the report
  fprintf(fp_out, "All Taxa\n");

  fprintf(fp_out, "\n");

  for (node_i = 0; node_i < nt->num_unique_nodes; node_i++)
  {
    nd = nt->NodeArray[node_i];

    if (nd->temporary == 1)
    {
      fprintf(fp_out, "temp node for internal processing internal id %d\n", nd->internal_node_id);
    }
    else
    {
      if ( (nd != nt->root) && (nd->reticulate == 0) )
      {
        fprintf(fp_out, "external id: %d internal id: %d intp1:%d\n", nd->external_node_id, nd->internal_node_id, nd->parent1_node->internal_node_id);
      } 
      else if (nd->reticulate == 1)
      {
        fprintf(fp_out, "external id: %d internal id: %d intp1:%d intp2:%d retic status %d\n", nd->external_node_id, nd->internal_node_id, nd->parent1_node->internal_node_id, nd->parent2_node->internal_node_id, nd->reticulate);
      }
      else
      {
        fprintf(fp_out, "external id: %d internal id: %d retic status %d\n", nd->external_node_id, nd->internal_node_id, nd->reticulate);
      }

      for (hchromo_i = 0; hchromo_i < num_hom_chromos; hchromo_i++)
      {
        for (strand_i = 0; strand_i < num_strands_per_hom_chrom; strand_i++)
        {
          fprintf(fp_out, "%s\n", nd->hom_chromos[hchromo_i]->strands[strand_i]->sequence);
        }
      }
    }

    fprintf(fp_out, "\n");
  }

  return;
}


/******************************************************************************/
/*                             PrintNewick                                    */
/******************************************************************************/
void PrintNewick(NODE treenode, int id_opt, int bl_opt)
{
  NODE tmp_nd;
  int hd;
  
  /* the last characters ); must be put in by the calling routine */

  if (treenode != NULL)
  {
    /* child subtree */
    if (treenode->first_child_node != NULL)
    {
      fprintf(fp_out, "(");
      PrintNewick(treenode->first_child_node, id_opt, bl_opt);
    }
    /* leaf node */
    else if ((treenode->extant_leaf == 1) &&
             (treenode->marker == 0) &&     // has not been written already
             (treenode->reticulate == 0) ) // guarantee of only 1 parent
    {
      if (id_opt == 1)
      {
        if (bl_opt == 0)
	{
          fprintf(fp_out, "%d", treenode->internal_node_id);
	}
	else if (bl_opt == 1)
	{
	  hd = HammingDistanceTotal(treenode, treenode->parent1_node);
          fprintf(fp_out, "%d:%d", treenode->internal_node_id, hd);
	}
	else // bl_opt == 2
	{
          fprintf(fp_out, "%d:%f", treenode->internal_node_id, treenode->brlen1);
	}
      }
      else
      {
        if (bl_opt == 0)
	{
          fprintf(fp_out, "%d", treenode->external_node_id);
	}
	else if (bl_opt == 1)
	{
	  hd = HammingDistanceTotal(treenode, treenode->parent1_node);
          fprintf(fp_out, "%d:%d", treenode->external_node_id, hd);
	}
	else  // bl_opt == 2
	{
          fprintf(fp_out, "%d:%f", treenode->external_node_id, treenode->brlen1);
	}
      }

      treenode->marker = 1;
    }

    /* root of (sub)tree */
    if ((treenode->first_child_node != NULL) && 
        (treenode->parent1_node != NULL))
    {
      if (id_opt == 1)
      {
        if (bl_opt == 0)
	{
          fprintf(fp_out, ")%d", treenode->internal_node_id);
	}
	else if (bl_opt == 1)
	{
	  hd = HammingDistanceTotal(treenode, treenode->parent1_node);
          fprintf(fp_out, ")%d:%d", treenode->internal_node_id, hd);
	}
	else // bl_opt == 2
	{
          fprintf(fp_out, ")%d:%f", treenode->internal_node_id, treenode->brlen1);
	}
      }
      else
      {
        if (bl_opt == 0)
	{
          fprintf(fp_out, ")%d", treenode->internal_node_id);
	}
	else if (bl_opt == 1)
	{
	  hd = HammingDistanceTotal(treenode, treenode->parent1_node);
          fprintf(fp_out, ")%d:%d", treenode->external_node_id, hd);
	}
	else // bl_opt == 2
	{
          fprintf(fp_out, ")%d:%f", treenode->external_node_id, treenode->brlen1);
	}
      }

      treenode->marker = 1;
    }

    // do the sibs now
    tmp_nd = treenode->next_sib_node;

    while(tmp_nd != NULL)
    {
      if (tmp_nd->marker == 0)
      {
        fprintf(fp_out, ",");
        PrintNewick(tmp_nd, id_opt, bl_opt);
      }
      tmp_nd = tmp_nd->next_sib_node;
    }
  }

  return;
}

/******************************************************************************/
/*                           PrintModifiedNewick                              */
/******************************************************************************/
void PrintModifiedNewick(NODE treenode, int id_opt, int bl_opt)
{
  NODE tmp_nd;
  int hd;

  /* the last ); must be put in by the calling routine */

  if (treenode != NULL)
  {
    if ( (treenode->first_child_node != NULL) && (treenode->marker == 0) )
    {
      fprintf(fp_out, "(");

      if (treenode->first_child_node->reticulate != 0)
      {
        if (treenode->first_child_node->parent1_node == treenode)
	{
	  last_hyb_parent_hit = 1;
	}
        else if (treenode->first_child_node->parent2_node == treenode)
	{
	  last_hyb_parent_hit = 2;
	}
	else
	{
	  printf("NR ERROR:  Problem with hybrid node in PrintMofidedNewick -- program aborted\n");
	  exit(0);
	}
      }

      PrintModifiedNewick(treenode->first_child_node, id_opt, bl_opt);
    }

    // if we have a child leaf node, or a reticulate node acting 
    // as a leaf because it has been printed once already, we want to 
    // print out the "leaf"
    else if ( ((treenode->extant_leaf == 1) && 
	       (treenode->reticulate == 0) &&
	       (treenode->marker == 0))||
	      ((treenode->reticulate != 0) && (treenode->marker == 1) &&
	         ((treenode->parent1_node->first_child_node == treenode) && 
		  (treenode->parent2_node->first_child_node == treenode))) )
    {
      if (treenode->reticulate == 1)
      {
        if (id_opt == 1)
	{
	  if (bl_opt == 0)
	  {
            fprintf(fp_out, "%d#H", treenode->internal_node_id);
	  }
	  else if (bl_opt == 1)
	  {
            fprintf(fp_out, "%d#H:0", treenode->internal_node_id);
	  }
	  else  // bl_opt == 2
	  {
            fprintf(fp_out, "%d#H:0", treenode->internal_node_id);
	  }
	}
	else
	{
	  if (bl_opt == 0)
	  {
            fprintf(fp_out, "%d#H", treenode->external_node_id);
	  }
	  else if (bl_opt == 1)
	  {
            fprintf(fp_out, "%d#H:0", treenode->external_node_id);
	  }
	  else // bl_opt == 2 
	  {
            fprintf(fp_out, "%d#H:0", treenode->external_node_id);
	  }
	}
      }
      else if (treenode->reticulate == 2)
      {
        if (id_opt == 1)
	{
	  if (bl_opt == 0)
	  {
            fprintf(fp_out, "%d#LGT", treenode->internal_node_id);
	  }
	  else if (bl_opt == 1)
	  {
            fprintf(fp_out, "%d#LGT:0", treenode->internal_node_id);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out, "%d#LGT:0", treenode->internal_node_id);
	  }
	}
	else
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out, "%d#LGT", treenode->external_node_id);
	  }
	  else if (bl_opt == 1)
	  {
            fprintf(fp_out, "%d#LGT:0", treenode->external_node_id);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out, "%d#LGT:0", treenode->external_node_id);
	  }
	}
      }
      else
      {
        if (id_opt == 1)
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out, "%d", treenode->internal_node_id);
	  }
	  else if (bl_opt == 1)
	  {
	    hd = HammingDistanceTotal(treenode, treenode->parent1_node);
            fprintf(fp_out, "%d:%d", treenode->internal_node_id, hd);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out, "%d:%f", treenode->internal_node_id, treenode->brlen1);
	  }
	}
	else
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out, "%d", treenode->external_node_id);
	  }
	  else if (bl_opt == 1)
	  {
	    hd = HammingDistanceTotal(treenode, treenode->parent1_node);
            fprintf(fp_out, "%d:%d", treenode->external_node_id, hd);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out, "%d:%f", treenode->external_node_id, treenode->brlen1);
	  }
	}
      }

      // mark this leaf/node as printed
      // (this marker goes up to 2, reticulate nodes get printed twice)
      treenode->marker = treenode->marker + 1;

    }

    /**********************************/
    /*             root               */
    /**********************************/
    /* root of (sub)tree */
    if ( (treenode->extant_leaf != 1) && (treenode->parent1_node != NULL) &&
	 (treenode->marker == 0) ) 
    {
      if (treenode->reticulate == 1)
      {
        if (id_opt == 1)
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out,")%d#H",treenode->internal_node_id);
	  }
	  else if (bl_opt == 1)
	  {
            fprintf(fp_out,")%d#H:0",treenode->internal_node_id);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out,")%d#H:0",treenode->internal_node_id);
	  }
	}
	else
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out,")%d#H",treenode->external_node_id);
	  }
	  else if (bl_opt == 1)
	  {
            fprintf(fp_out,")%d#H:0",treenode->external_node_id);
	  }
	  else  // bl_opt == 2
	  {
            fprintf(fp_out,")%d#H:0",treenode->external_node_id);
	  }
	}
      }
      else if (treenode->reticulate == 2)
      {
        if (id_opt == 1)
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out,")%d#LGT",treenode->internal_node_id);
	  }
	  else if (bl_opt == 1)
          {
            fprintf(fp_out,")%d#LGT:0",treenode->internal_node_id);
	  }
	  else  // bl_opt == 2
          {
            fprintf(fp_out,")%d#LGT:0",treenode->internal_node_id);
	  }
	}
	else
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out,")%d#LGT",treenode->external_node_id);
	  }
	  else if (bl_opt == 1)
	  {
            fprintf(fp_out,")%d#LGT:0",treenode->external_node_id);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out,")%d#LGT:0",treenode->external_node_id);
	  }
	}
      }
      else
      {
        if (id_opt == 1)
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out,")%d",treenode->internal_node_id);
	  }
	  else if (bl_opt == 1)
	  {
	    hd = HammingDistanceTotal(treenode, treenode->parent1_node);
            fprintf(fp_out,")%d:%d",treenode->internal_node_id, hd);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out,")%d:%f",treenode->internal_node_id, treenode->brlen1);
	  }
	}
	else
	{
	  if (bl_opt == 0)
          {
            fprintf(fp_out,")%d",treenode->external_node_id);
	  }
	  else if (bl_opt == 1)
	  {
	    hd = HammingDistanceTotal(treenode, treenode->parent1_node);
            fprintf(fp_out,")%d:%d",treenode->external_node_id, hd);
	  }
	  else // bl_opt == 2
	  {
            fprintf(fp_out,")%d:%f",treenode->external_node_id, treenode->brlen1);
	  }
	}
      }

      // mark this leaf/node as printed
      // (this marker goes up to 2, reticulate nodes get printed twice)
      treenode->marker = treenode->marker + 1;
    }

    /**********************************/
    /*      do the siblings here      */
    /**********************************/

    if (treenode->reticulate == 1)
    {
      if (last_hyb_parent_hit == 1)
      {
        tmp_nd = treenode->next_p1_sib;
      }
      else if (last_hyb_parent_hit == 2)
      {
        tmp_nd = treenode->next_p2_sib;
      }
      else
      {
        printf("NR ERROR:  Unexpected problem with printing modified newick (treenode internal id %d with marker %d  last hyb parent hit %d) -- program aborted\n", treenode->internal_node_id, treenode->marker, last_hyb_parent_hit);
	exit(0);
      }
    }
    else // regular, non-reticulate node
    {
      tmp_nd = treenode->next_sib_node;
    }

    while(tmp_nd != NULL)
    {
      if (tmp_nd->marker == 0)
      {
        fprintf(fp_out, ",");
        PrintModifiedNewick(tmp_nd, id_opt, bl_opt);
      }
      tmp_nd = tmp_nd->next_sib_node;
    }


  } // treenode is not NULL

  return;

}

