/*****************************************************************************/
/*                                                                           */
/*  NetReconstruct kappa version                                             */
/*                                                                           */
/*  Software to perform phylogenetic network reconstructions when            */
/*  diploid hybrids are part of the evolutionary history.                    */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<math.h>
#include<float.h>
#include<limits.h>
#include"globals.h"
#include"node.h"
#include"network.h"
#include"twister.h"
#include"randipinter.h"


/******************************************************************************/
/*                            ExecuteRandIP                                   */
/******************************************************************************/
void ExecuteRandIP(NETWORK nt, int subtree_id)
{
  FILE *pipe_ptr;
  char exec_rip[500];

  unsigned long rand_val;
  char rand_num_seed_str[11];

  int max_rip_tries;
  char max_rip_tries_str[11];

  // start the command
  strcpy(exec_rip, "./RIP -i phylip_infile -o outtree -o outfile -r ");

  // add the random seed
  rand_val = genrand_int32();
  sprintf(rand_num_seed_str, "%lu", rand_val);
  strcat(exec_rip, rand_num_seed_str);

  // add max tries
  strcat(exec_rip, " -z ");
  max_rip_tries = 1000 * num_extant_total;
  sprintf(max_rip_tries_str, "%d", max_rip_tries);
  strcat(exec_rip, max_rip_tries_str);

  // add rc capture
  strcat(exec_rip, " > rip.rc");

  if ((pipe_ptr = popen(exec_rip, "r")) != NULL)
  {
    // close the pipe
    (void) pclose(pipe_ptr);
  }
  else
  {
    printf("NR ERROR:  Unexpected issue when running RANDIP -- program aborted\n");
    exit(0);
  }

  return;
}

