/*****************************************************************************/
/*                                                                           */
/*  RANDIP kappa version                                                     */
/*                                                                           */
/*  Software to perform random tree reconstruction for use with NR code.     */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <float.h>
#include <ctype.h>
#include <curses.h>
#include "globals.h"
#include "node.h"
#include "tree.h"
#include "output.h"
#include "input.h"

/*****************************************************************************/
/*                         Read the input file!                              */
/*****************************************************************************/
void ReadInput(TREE tr) 
{
  char str[MAX_LINE_LENGTH];
  int junk_seq_length;
  int taxon_id;
  int line_i;


  // set all default value
  num_extant_total = 0;

  fscanf(fp_in, "%d", &num_extant_total);
  fscanf(fp_in, "%d", &junk_seq_length);

  if (junk_seq_length > MAX_SEQ_LENGTH)
  {
    printf("RIP ERROR:  Although sequence are not used by RIP, the provided sequence length is too long for the input routine to skip -- program aborted\n");
    exit(0);
  }

  // read the extant taxa ids and check them
  for (line_i = 0; line_i < num_extant_total; line_i++)
  {
    fscanf(fp_in, "%d", &taxon_id);
    tr->NodeArray[tr->node_id_ctr] = NewNode(tr->node_id_ctr);

    if (taxon_id <= MAX_NODE_ID)
    {
      tr->NodeArray[tr->node_id_ctr]->external_node_id = taxon_id;
    }
    else
    {
      printf("RIP ERROR:  Taxon id (%d) exceeds the limit (%d) for external id values -- program aborted\n", taxon_id, MAX_NODE_ID);
      exit(0);
    }
    
    // remember this is an extant leaf
    tr->NodeArray[tr->node_id_ctr]->extant_leaf = 1;
    
    // up the necesary counters
    tr->node_id_ctr++;
    tr->num_unique_nodes++;
    tr->num_unique_extant_leaves++;

    // read the rest of the line but toss it because 
    // we do not care about sequence information
    fscanf(fp_in, "%s", str);

    if (strchr(str, '\n') == NULL)
    {
      // no end of line found so do getline and ignore rest of line
      // (this should not happen for these input files as a seq
      // should follow the id
      fgets(str, MAX_LINE_LENGTH, fp_in);
    }
    else
    {
      // end of line was found, so do nothing but proceed with processing
      ;
    }
  }

  // mark the very first one as the outgroup -- this is the enforced
  // default we use for PHYLIP so do the same here and then we can
  // do the rest of the reconstruction accordingly
  tr->NodeArray[0]->outgroup = 1;

  if (num_extant_total > MAX_LEAVES)
  {
    printf("RIP ERROR:  Maximum number of leaves (%d) cannot be exceeded -- program aborted\n", MAX_LEAVES);
    exit(0);
  }

  return;
}
