/*****************************************************************************/
/*                                                                           */
/*  RANDIP kappa version                                                     */
/*                                                                           */
/*  Software to perform random tree reconstruction for use with NR code.     */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "node.h"
#include "tree.h"
#include "input.h"
#include "output.h"
#include "randrecon.h"
#include "twister.h"

/*****************************************************************************/
/*                                  main                                     */
/*****************************************************************************/

int main(const int argc, const char *argv[])
{
  TREE tr;

  /****************************************************************************/
  /* the command line is:                                                     */
  /*                                                                          */
  /* 'RIP -i <inputfile> -o <outtreefile> -o <outfile> -r <randseed> -z <maxtries>' */
  /****************************************************************************/

  if (argc != 11)
  {
    printf("RIP ERROR: Command line is:  'RIP -i <inputfile> -o <outtreefile> -o <outfile> -r <randseed> -z <maxtries>'\n");
    exit(0);
  };

  if (strcmp(argv[1], "-i")!=0 || 
             strcmp(argv[3],"-o")!=0 || 
             strcmp(argv[5],"-o")!=0 || 
             strcmp(argv[7],"-r")!=0 || 
	     strcmp(argv[9], "-z")!=0)
  {
    printf("RIP ERROR: Command line is:  'RIP -i <inputfile> -o <outtreefile> -o <outfile> -r <randseed> -z <maxtries>'\n");
    exit(0);
  }

  strcpy(infile_name, argv[2]);

  // open the input/output files
  fp_in = fopen(argv[2], "r");
  if (fp_in == NULL)
  {
    printf("RIP ERROR: Can't open the input file '%s'\n", argv[2]);
    exit(0);
  }

  fp_outtree = fopen(argv[4], "w");
  if (fp_outtree == NULL)
  {
    printf("RIP ERROR: Can't open the outtree file '%s'\n", argv[4]);
    exit(0);
  }

  fp_outfile = fopen(argv[6], "w");
  if (fp_outfile == NULL)
  {
    printf("RIP ERROR: Can't open the outfile file '%s'\n", argv[6]);
    exit(0);
  }

  // set the seed for the random numbers
  sscanf(argv[8], "%lu", &input_seed);
  init_genrand(input_seed);

  // set the max_tries
  sscanf(argv[10], "%lu", &max_tries);

  // first make the basics of the tree
  tr = NewTree();

  // read the input
  ReadInput(tr);

  // now do the important work
  RandReconTree(tr);

  // write the output
  ReconReport(tr);
  
  // be good and clean up
  fclose(fp_in);
  fclose(fp_outtree);
  fclose(fp_outfile);


  return 1;
}
