/*****************************************************************************/
/*                                                                           */
/*  RANDIP kappa version                                                     */
/*                                                                           */
/*  Software to perform random tree reconstruction for use with NR code.     */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<math.h>
#include<limits.h>
#include<float.h>
#include"globals.h"
#include"node.h"


/******************************************************************************/
/*                                 NewNode                                    */
/******************************************************************************/
NODE NewNode(int nd_id) 
{
  NODE nd;
  
  // make sure this node will not exceed our limit
  if (nd_id >= (MAX_NODES))
  {
    printf("RIP ERROR: Simulation could not proceed as MAX_NODES (%d) is exceeded\n", MAX_NODES);
    exit(0);
  }

  // make the new node
  nd = (NODE)malloc(sizeof(NODETYPE));

  if (nd == NULL)
  {
    printf("RIP ERROR: Could not allocate space for node\n");
    exit(0);
  }

  // use the values we were given
  nd->internal_node_id = nd_id;

  // set all others to default values
  nd->external_node_id = -1;
  nd->left = NULL;
  nd->right = NULL;
  nd->parent = NULL;

  nd->extant_leaf = 0;   
  nd->outgroup = 0;
  nd->avail_marker = -1;
  nd->marker = 0;

  return nd;

}  // end of NewNode()
