/*****************************************************************************/
/*                                                                           */
/*  RANDIP kappa version                                                     */
/*                                                                           */
/*  Software to perform random tree reconstruction for use with NR code.     */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<math.h>
#include"globals.h"
#include"node.h"
#include"tree.h"
#include"output.h"


/******************************************************************************/
/*                              ReconReport                                   */
/******************************************************************************/
void ReconReport(TREE tr)
{
  int id_opt;

  id_opt = 1; // 0 = internal, 1 = external (which we want for real output file)

  // REMEBER...each report has which output file it goes to hardcoded
  
  // OUTTREE STUFF
  // this is where we do the topology -- we do not seem to use the marker
  // with this simple data structure, but just be safe and reset anyway
  ResetMarkersAll(tr);

  PrintNewick(tr->root, id_opt);

  // termination for newick format
  if (id_opt == 0)
    fprintf(fp_outtree, ")%d;\n", tr->root->internal_node_id);
  else
    fprintf(fp_outtree, ");\n");

  ResetMarkersAll(tr);

  // OUTFILE STUFF
  fprintf(fp_outfile, "This is additional information from RANDIP (RIP) run.\n");
  fprintf(fp_outfile, "\n");
  fprintf(fp_outfile, "The inputted seed for this run was: %lu\n", input_seed);
  fprintf(fp_outfile, "The max tries limit for this run was: %lu\n", max_tries);
  fprintf(fp_outfile, "\n");

  InputtedExtantTaxa(tr);
  AllTaxa(tr);

  return;
  
} 

/******************************************************************************/
/*                            InputtedExtantTaxa                              */
/******************************************************************************/
void InputtedExtantTaxa(TREE tr)
{
  int node_i;
  NODE nd;

  fprintf(fp_outfile, "\n");
  fprintf(fp_outfile, "\n");

  // starting the basics of the report
  fprintf(fp_outfile, "Inputted Extant Taxa\n");
  fprintf(fp_outfile, "------------------------\n");

  fprintf(fp_outfile, "\n");

  for (node_i = 0; node_i < tr->num_unique_nodes; node_i++)
  {
    nd = tr->NodeArray[node_i];

    if (nd->extant_leaf == 1)
    {
      if (nd->parent != NULL)
      {
        fprintf(fp_outfile, "internal id: %d  external_id: %d  (parent internal id: %d  parent external id: %d) \n", nd->internal_node_id, nd->external_node_id, nd->parent->internal_node_id, nd->parent->external_node_id);
      }
      else
      {
        fprintf(fp_outfile, "internal id: %d  external_id: %d \n", nd->internal_node_id, nd->external_node_id);
      }
    }

    fprintf(fp_outfile, "\n");
  }

  return;

}

/******************************************************************************/
/*                                AllTaxa                                     */
/******************************************************************************/
void AllTaxa(TREE tr)
{
  int node_i;
  NODE nd;

  fprintf(fp_outfile, "\n");
  fprintf(fp_outfile, "\n");

  // starting the basics of the report
  fprintf(fp_outfile, "All Taxa\n");
  fprintf(fp_outfile, "------------\n");

  fprintf(fp_outfile, "\n");

  for (node_i = 0; node_i < tr->num_unique_nodes; node_i++)
  {
    nd = tr->NodeArray[node_i];

    if (nd->parent != NULL) 
    {
      fprintf(fp_outfile, "internal id: %d  external_id: %d  (parent internal id: %d  parent external id: %d)", nd->internal_node_id, nd->external_node_id, nd->parent->internal_node_id, nd->parent->external_node_id);
    }
    else
    {
      fprintf(fp_outfile, "internal id: %d  external_id: %d  parent is NULL", nd->internal_node_id, nd->external_node_id);
    }

    if (nd->left != NULL) 
    {
      fprintf(fp_outfile, "  left child internal id: %d  left child external_id: %d", nd->left->internal_node_id, nd->left->external_node_id);
    }
    else
    {
      fprintf(fp_outfile, "  left child is NULL");
    }

    if (nd->right != NULL) 
    {
      fprintf(fp_outfile, "  right child internal id: %d  right child external_id: %d", nd->right->internal_node_id, nd->right->external_node_id);
    }
    else
    {
      fprintf(fp_outfile, "  right child is NULL");
    }

    fprintf(fp_outfile, "\n");
    fprintf(fp_outfile, "\n");
  }

  return;
}


/******************************************************************************/
/*                             PrintNewick                                    */
/******************************************************************************/
void PrintNewick(NODE treenode, int id_opt)
{
  /* the last characters ); must be put in by the calling routine */

  if (treenode != NULL)
  {
    /* left subtree */
    if (treenode->left != NULL)
    {
      fprintf(fp_outtree, "(");
      PrintNewick(treenode->left, id_opt);
    }
    /* left leaf node */
    else if ( (treenode->extant_leaf == 1) && 
              (treenode->parent->left == treenode) )
    {
      if (id_opt == 0)
      {
        fprintf(fp_outtree, "%d", treenode->internal_node_id);
      }
      else 
      {
        fprintf(fp_outtree, "%d", treenode->external_node_id);
      }
    }

    /* right subtree */
    if (treenode->right != NULL)
    {
      fprintf(fp_outtree, ",");
      PrintNewick(treenode->right, id_opt);
    }
    /* right leaf node */
    else if ( (treenode->extant_leaf == 1) &&
              (treenode->parent->right == treenode) )
    {
      if (id_opt == 0)
      {
        fprintf(fp_outtree, "%d", treenode->internal_node_id);
      }
      else
      {
        fprintf(fp_outtree, "%d", treenode->external_node_id);
      }
    }

    /* root of (sub)tree  -- don't print external ids*/
    if ((treenode->left != NULL) && (treenode->right != NULL) &&
	(treenode->parent != NULL))
    {
      if (id_opt == 0)
      {
        fprintf(fp_outtree,")%d", treenode->internal_node_id);
      }
      else
      {
        fprintf(fp_outtree,")");
      }
    }
  }
  
  return;
}
