/*****************************************************************************/
/*                                                                           */
/*  RANDIP kappa version                                                     */
/*                                                                           */
/*  Software to perform random tree reconstruction for use with NR code.     */
/*                                                                           */
/*  Copyright (C) 2007 Monique M. Morin                                      */
/*                                                                           */
/*  This program is free software; you can redistribute it and/or modify it  */
/*  under the terms of the GNU General Public License as published by the    */
/*  Free Software Foundation; either version 2 of the License, or (at your   */
/*  option) any later version.                                               */
/*                                                                           */
/*  This program is distributed in the hope that it will be useful, but      */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of               */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Public */
/*  License for more details.                                                */
/*                                                                           */
/*  You should have received a copy of the GNU General Public License along  */
/*  with this program; if not, write to the Free Software Foundation, Inc.,  */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                    */
/*                                                                           */
/*  Monique Morin (morin@cs.unm.edu)                                         */
/*                                                                           */
/*  Monique Morin                                                            */
/*  University of New Mexico                                                 */
/*  Department of Computer Science                                           */
/*  Mail stop: MSC01 1130                                                    */
/*  1 University of New Mexico                                               */
/*  Albuquerque, NM 87131-0001                                               */
/*****************************************************************************/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<float.h>
#include<limits.h>
#include<math.h>
#include<time.h>
#include<ctype.h>
#include"globals.h"
#include"node.h"
#include"tree.h"
#include"twister.h"

/******************************************************************************/
/*                                 NewTree                                    */
/******************************************************************************/

TREE NewTree() 
{
  TREE tr = (TREE)malloc(sizeof(TREETYPE));
 
  if (tr == NULL)
  {
    printf("NR ERROR: Could not allocate space for tree\n");
    exit(0);
  }
  
  tr->root = NULL;
  tr->node_id_ctr = 0;
  tr->num_unique_nodes = 0;
  tr->num_unique_extant_leaves = 0;

  return tr;
}


/******************************************************************************/
/*                              CleanUpMemory                                 */
/******************************************************************************/
void CleanUpMemory(TREE tr)
{
  long node_i;

  // first clean up the tree stuff
  for (node_i = 0; node_i < (tr->num_unique_nodes); node_i++)
  {
    free(tr->NodeArray[node_i]);
  }

  // now we are free to free the tree -- and it has no dynamically
  // allocated structures left so just go ahead and free it!
  free(tr);

  return;
}

/******************************************************************************/
/*                          ResetMarkersAll                                   */
/******************************************************************************/
void ResetMarkersAll(TREE tr)
{
  int node_i;

  // This routine resets ALL node markers by going through the node array

  for (node_i = 0; node_i < tr->num_unique_nodes; node_i++)
  {
    tr->NodeArray[node_i]->marker = 0;
  }

  return;

}

/******************************************************************************/
/*                          ResetAvailMarkersAll                              */
/******************************************************************************/
void ResetAvailMarkersAll(TREE tr, int val)
{
  int node_i;

  // This routine resets ALL node markers by going through the node array

  for (node_i = 0; node_i < tr->num_unique_nodes; node_i++)
  {
    tr->NodeArray[node_i]->avail_marker = val;
  }

  return;

}
